Package: GenomicRanges
Title: Representation and manipulation of genomic intervals
Description: The ability to efficiently represent and manipulate genomic
	annotations and alignments is playing a central role when it comes
	to analyzing high-throughput sequencing data (a.k.a. NGS data).
	The GenomicRanges package defines general purpose containers for
	storing and manipulating genomic intervals and variables defined along
	a genome. More specialized containers for representing and manipulating
	short alignments against a reference genome, or a matrix-like
	summarization of an experiment, are defined in the GenomicAlignments
	and SummarizedExperiment packages, respectively. Both packages build
	on top of the GenomicRanges infrastructure.
biocViews: Genetics, Infrastructure, DataRepresentation, Sequencing,
        Annotation, GenomeAnnotation, Coverage
URL: https://bioconductor.org/packages/GenomicRanges
BugReports: https://github.com/Bioconductor/GenomicRanges/issues
Version: 1.64.0
License: Artistic-2.0
Encoding: UTF-8
Authors@R: c(
	person("Patrick", "Aboyoun", role="aut"),
	person("Hervé", "Pagès", role=c("aut", "cre"),
		email="hpages.on.github@gmail.com"),
	person("Michael", "Lawrence", role="aut"),
	person("Sonali", "Arora", role="ctb"),
	person("Martin", "Morgan", role="ctb"),
	person("Kayla", "Morrell", role="ctb"),
	person("Valerie", "Obenchain", role="ctb"),
	person("Marcel", "Ramos", role="ctb"),
	person("Lori", "Shepherd", role="ctb"),
	person("Dan", "Tenenbaum", role="ctb"),
	person("Daniel", "van Twisk", role="ctb"))
Depends: R (>= 4.0.0), methods, stats4, BiocGenerics (>= 0.53.2),
        S4Vectors (>= 0.45.2), IRanges (>= 2.43.6), Seqinfo (>= 0.99.3)
Imports: utils, stats
Suggests: GenomeInfoDb, Biobase, AnnotationDbi, annotate, Biostrings
        (>= 2.77.2), SummarizedExperiment (>= 1.39.1), Rsamtools,
        GenomicAlignments, BSgenome, GenomicFeatures, UCSC.utils,
        txdbmaker, Gviz, VariantAnnotation, AnnotationHub, DESeq2,
        DEXSeq, edgeR, KEGGgraph, RNAseqData.HNRNPC.bam.chr14,
        pasillaBamSubset, KEGGREST, hgu95av2.db, hgu95av2probe,
        BSgenome.Scerevisiae.UCSC.sacCer2, BSgenome.Hsapiens.UCSC.hg38,
        BSgenome.Mmusculus.UCSC.mm10,
        TxDb.Athaliana.BioMart.plantsmart51,
        TxDb.Dmelanogaster.UCSC.dm3.ensGene,
        TxDb.Hsapiens.UCSC.hg38.knownGene,
        TxDb.Mmusculus.UCSC.mm10.knownGene, RUnit, digest, knitr,
        rmarkdown, BiocStyle
VignetteBuilder: knitr
Collate: normarg-utils.R utils.R phicoef.R constraint.R strand-utils.R
        genomic-range-squeezers.R GenomicRanges-class.R
        GenomicRanges-comparison.R GRanges-class.R GPos-class.R
        GRangesFactor-class.R DelegatingGenomicRanges-class.R
        GNCList-class.R GenomicRangesList-class.R GRangesList-class.R
        makeGRangesFromDataFrame.R makeGRangesListFromDataFrame.R
        RangedData-methods.R findOverlaps-methods.R
        intra-range-methods.R inter-range-methods.R coverage-methods.R
        setops-methods.R subtract-methods.R nearest-methods.R
        absoluteRanges.R tileGenome.R tile-methods.R genomicvars.R
        zzz.R
git_url: https://git.bioconductor.org/packages/GenomicRanges
git_branch: RELEASE_3_23
git_last_commit: ac531f1
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:18:39 UTC; biocbuild
Author: Patrick Aboyoun [aut],
  Hervé Pagès [aut, cre],
  Michael Lawrence [aut],
  Sonali Arora [ctb],
  Martin Morgan [ctb],
  Kayla Morrell [ctb],
  Valerie Obenchain [ctb],
  Marcel Ramos [ctb],
  Lori Shepherd [ctb],
  Dan Tenenbaum [ctb],
  Daniel van Twisk [ctb]
Maintainer: Hervé Pagès <hpages.on.github@gmail.com>
