# GenomicRanges

Python equivalent to Bioconductor's [GenomicRanges](https://bioconductor.org/packages/release/bioc/html/GenomicRanges.html) to represent genomic locations and support genomic analysis. It uses efficient structures already available in the Pandas/NumPy eco-system.

## Install

Package is deployed to [PyPI](https://pypi.org/project/genomicranges/)

```shell
pip install genomicranges
```

## Contents

```{toctree}
:maxdepth: 2

Overview <readme>
Tutorial <tutorial>
Module Reference <api/modules>
Contributions & Help <contributing>
License <license>
Authors <authors>
Changelog <changelog>
```

## Indices and tables

- {ref}`genindex`
- {ref}`modindex`
- {ref}`search`

[Sphinx]: http://www.sphinx-doc.org/
[Markdown]: https://daringfireball.net/projects/markdown/
[reStructuredText]: http://www.sphinx-doc.org/en/master/usage/restructuredtext/basics.html
[recommonmark]: https://recommonmark.readthedocs.io/en/latest
[autostructify]: https://recommonmark.readthedocs.io/en/latest/auto_structify.html
